@@ -21,18 +21,24 @@
 
 """Classes and functions on collections of backup volumes"""
 
-import os
 import gzip
 import json
+import os
 
-from duplicity import config
-from duplicity import dup_time
-from duplicity import file_naming
-from duplicity import log
-from duplicity import manifest
-from duplicity import path
-from duplicity import util
+from duplicity import (
+    config,
+    dup_time,
+    file_naming,
+    log,
+    manifest,
+    path,
+    util,
+)
 from duplicity.gpg import GPGError
+from duplicity.log import (
+    InfoCode,
+    Log,
+)
 
 
 class CollectionsError(Exception):
@@ -48,7 +54,8 @@ class BackupSet(object):
         """
         Initialize new backup set, only backend is required at first
         """
-        self.backend = backend
+        self.type = None  # full or inc
+        self.backend = backend  # backend to use
         self.info_set = False  # true if fields are set
         self.volume_name_dict = {}  # dict from volume number to filename
         self.remote_manifest_name = None  # full name of remote manifest
@@ -60,7 +67,9 @@ class BackupSet(object):
         self.end_time = None  # will be set if inc
         self.partial = False  # true if a partial backup
         self.encrypted = False  # true if an encrypted backup
-        self.files_changed = []
+        self.files_changed = []  # list of files changed
+        self.mf_missing = {}  # vols missing from manifest
+        self.cs_missing = {}  # vols missing from collection
 
     def is_complete(self):
         """
@@ -68,6 +77,29 @@ class BackupSet(object):
         """
         return self.remote_manifest_name
 
+    def is_missing(self):
+        """
+        Assume missing if manifest entries or difftar volumes are missing
+        """
+        return self.mf_missing or self.cs_missing
+
+    def get_missing(self):
+        """
+        check that we have a complete set of volumes.
+        """
+        if not config.check_remote:
+            self.mf_missing = {}
+            self.cs_missing = {}
+            return
+        try:
+            mf = self.get_manifest()
+        except Exception as e:
+            return
+        mf_set = set(mf.volume_info_dict.keys())
+        cs_set = set(self.volume_name_dict.keys())
+        self.mf_missing = cs_set - mf_set
+        self.cs_missing = mf_set - cs_set
+
     def add_filename(self, filename, pr=None):
         """
         Add a filename to given set.  Return true if it fits.
@@ -106,7 +138,9 @@ class BackupSet(object):
         elif pr.manifest:
             self.set_manifest(filename)
         else:
-            assert pr.volume_number is not None
+            assert (
+                pr.volume_number is not None
+            ), f"Missing volume_number in ParseResults for file {os.fsdecode(filename)}"
             assert pr.volume_number not in self.volume_name_dict, (
                 f"Volume {int(pr.volume_number)} is already in the volume list as "
                 f"{os.fsdecode(self.volume_name_dict[pr.volume_number])}. "
@@ -124,7 +158,7 @@ class BackupSet(object):
         @param pr: parse results
         @type pr: ParseResults
         """
-        assert not self.info_set
+        assert not self.info_set, "BackupSet info already set; cannot set twice"
         self.type = pr.type
         self.time = pr.time
         self.start_time = pr.start_time
@@ -166,9 +200,6 @@ class BackupSet(object):
         """
         Add local and remote manifest filenames to backup set
         """
-        assert (
-            not self.remote_manifest_name
-        ), f"Cannot set filename of remote manifest to {remote_filename}; already set to {self.remote_manifest_name}."
         self.remote_manifest_name = remote_filename
 
         local_filename_list = config.archive_dir_path.listdir()
@@ -261,7 +292,7 @@ class BackupSet(object):
                 remote_manifest = local_manifest
             else:
                 log.FatalError(
-                    _("Fatal Error: Neither remote nor local " "manifest is readable."),
+                    _("Fatal Error: Neither remote nor local manifest is readable."),
                     log.ErrorCode.unreadable_manifests,
                 )
         remote_manifest.check_dirinfo()
@@ -270,17 +301,23 @@ class BackupSet(object):
         """
         Return manifest object by reading local manifest file
         """
-        assert self.local_manifest_path
+        assert self.local_manifest_path, "Local manifest path is not set for this BackupSet"
         manifest_buffer = self.local_manifest_path.get_data()
-        log.Info(_("Processing local manifest %s (%s)") % (self.local_manifest_path.uc_name, len(manifest_buffer)))
+        log.Info(_(f"Processing local manifest {self.local_manifest_path.uc_name} ({len(manifest_buffer)})"))
         return manifest.Manifest().from_string(manifest_buffer)
 
     def get_remote_manifest(self):
         """
         Return manifest by reading remote manifest on backend
         """
-        assert self.remote_manifest_name
-        manifest_buffer = self.get_remote_file(self.remote_manifest_name)
+        if config.check_remote:
+            assert self.remote_manifest_name, "No remote manifest name"
+            manifest_buffer = self.get_remote_file(self.remote_manifest_name)
+            log.Info(_(f"Processing remote manifest {self.remote_manifest_name} ({len(manifest_buffer)})"))
+        else:
+            assert self.local_manifest_path, "No local manifest path"
+            manifest_buffer = self.local_manifest_path.get_data()
+            log.Info(_(f"Using local manifest as remote {self.local_manifest_path.uc_name} ({len(manifest_buffer)})"))
         return manifest.Manifest().from_string(manifest_buffer)
 
     def get_remote_file(self, remote_file):
@@ -304,10 +341,10 @@ class BackupSet(object):
     def get_jsonstat(self):
         if self.local_jsonstat_path:
             json_stat_bytes = gzip.decompress(self.local_jsonstat_path.get_data())
-        elif self.remote_jsonstat_name:
+        elif self.remote_jsonstat_name and config.check_remote:
             json_stat_bytes = self.get_remote_file(self.remote_jsonstat_name)
         else:
-            log.Info(_("No Jsonstat file found, return empty."))
+            log.Info(_("No jsonstat file found, return empty."))
             return {}
         return json.loads(json_stat_bytes)
 
@@ -315,7 +352,7 @@ class BackupSet(object):
         """
         Return sorted list of (remote) filenames of files in set
         """
-        assert self.info_set
+        assert self.info_set, "BackupSet info must be set before retrieving filenames"
         volume_num_list = sorted(self.volume_name_dict.keys())
         volume_filenames = [self.volume_name_dict[x] for x in volume_num_list]
         if self.remote_manifest_name:
@@ -383,9 +420,11 @@ class BackupChain(object):
         """
         Add full backup set
         """
-        assert not self.fullset and isinstance(fullset, BackupSet)
+        assert not self.fullset and isinstance(
+            fullset, BackupSet
+        ), "Full backup set must be unset and argument must be a BackupSet instance"
         self.fullset = fullset
-        assert fullset.time
+        assert fullset.time, "Full backup set must have a valid 'time' attribute"
         self.start_time, self.end_time = fullset.time, fullset.time
 
     def add_inc(self, incset):
@@ -419,7 +458,7 @@ class BackupChain(object):
                 dup_time.timetopretty(incset.end_time),
             )
         )
-        assert self.end_time
+        assert self.end_time, "BackupChain end_time must be set after adding an incremental set"
         return True
 
     def delete(self, keep_full=False):
@@ -616,7 +655,7 @@ class SignatureChain(object):
         Return ordered list of signature fileobjs opened for reading,
         optionally at a certain time
         """
-        assert self.fullsig
+        assert self.fullsig, "SignatureChain fullsig must be set before accessing related data"
         if self.archive_dir_path:  # local
 
             def filename_to_fileobj(filename):
@@ -639,7 +678,7 @@ class SignatureChain(object):
             if not keep_full:
                 self.archive_dir_path.append(self.fullsig).delete()
         else:
-            assert self.backend
+            assert self.backend, "Backend must be set before performing this operation"
             inclist_copy = self.inclist[:]
             inclist_copy.reverse()
             if not keep_full:
@@ -668,12 +707,13 @@ class CollectionsStatus(object):
     Hold information about available chains and sets
     """
 
-    def __init__(self, backend, archive_dir_path):
+    def __init__(self, backend, archive_dir_path, first=False):
         """
         Make new object.  Does not set values
         """
         self.backend = backend
         self.archive_dir_path = archive_dir_path
+        self.first = first
 
         # Will hold (signature chain, backup chain) pair of active
         # (most recent) chains
@@ -689,9 +729,11 @@ class CollectionsStatus(object):
         self.remote_orphaned_sig_names = []
         self.orphaned_backup_sets = None
         self.incomplete_backup_sets = None
+        self.missing_difftar_sets = None
+        self.last_chain_missing_difftars = False
 
         # True if set_values() below has run
-        self.values_set = None
+        self.values_set = False
 
     def to_log_info(self):
         """
@@ -734,7 +776,7 @@ class CollectionsStatus(object):
             l.append("")
 
         if self.matched_chain_pair:
-            l.append("\n" + _("Found primary backup chain with matching " "signature chain:"))
+            l.append("\n" + _("Found primary backup chain with matching signature chain:"))
             l.append(str(self.matched_chain_pair[1]))
         else:
             l.append(_("No backup chains with active signatures found"))
@@ -743,7 +785,7 @@ class CollectionsStatus(object):
             l.append(_("Also found %d backup set(s) not part of any chain,") % len(self.orphaned_backup_sets))
             l.append(_("and %d incomplete backup set(s).") % len(self.incomplete_backup_sets))
             # TRANSL: "cleanup" is a hard-coded command, so do not translate it
-            l.append(_("These may be deleted by running duplicity with the " '"cleanup" command.'))
+            l.append(_('These may be deleted by running duplicity with the "cleanup" command.'))
         else:
             l.append(_("No orphaned or incomplete backup sets found."))
 
@@ -757,16 +799,19 @@ class CollectionsStatus(object):
         do not warn about unnecessary sig chains.  This is because there may
         naturally be some unecessary ones after a full backup.
         """
-        self.values_set = 1
-
-        # get remote filename list
-        backend_filename_list = self.backend.list()
-        log.Debug(_("%d file(s) exists on backend") % len(backend_filename_list))
+        self.values_set = True
 
         # get local filename list
         local_filename_list = self.archive_dir_path.listdir()
         log.Debug(_("%d file(s) exist in cache") % len(local_filename_list))
 
+        # get remote filename list
+        if config.check_remote:
+            backend_filename_list = self.backend.list()
+        else:
+            backend_filename_list = local_filename_list
+        log.Debug(_("%d file(s) exists on backend") % len(backend_filename_list))
+
         # check for partial backups
         partials = []
         for local_filename in local_filename_list:
@@ -779,6 +824,7 @@ class CollectionsStatus(object):
             backup_chains,
             self.orphaned_backup_sets,
             self.incomplete_backup_sets,
+            self.missing_difftar_sets,
         ) = self.get_backup_chains(partials + backend_filename_list)
         backup_chains = self.get_sorted_chains(backup_chains)
         self.all_backup_chains = backup_chains
@@ -819,7 +865,7 @@ class CollectionsStatus(object):
                     and sig_chains[i].end_time == latest_backup_chain.get_all_sets()[-2].end_time
                 ):
                     # It matches, remove the last backup set:
-                    log.Warn(_("Warning, discarding last backup set, because " "of missing signature file."))
+                    log.Warn(_("WARNING. discarding last backup set, because of missing signature file."))
                     self.incomplete_backup_sets.append(latest_backup_chain.incset_list[-1])
                     latest_backup_chain.incset_list = latest_backup_chain.incset_list[:-1]
                 else:
@@ -838,11 +884,37 @@ class CollectionsStatus(object):
         """
         Log various error messages if find incomplete/orphaned files
         """
-        assert self.values_set
+        assert self.values_set, "CollectionsStatus values must be set (call set_values) before warning/reporting"
+
+        def missing_to_log_info(s):
+            """
+            Generate log information for missing volumes in the backup chain.
+            """
+            l = []
+            if s.is_missing():
+                l.append(
+                    f"WARNING: Backup set at {dup_time.timetostring(s.start_time)} has missing "
+                    f"volumes or manifest entries:"
+                )
+                if s.cs_missing:
+                    l.append(f"  - remote volumes: {[f'vol{mv}.difftar' for mv in s.cs_missing]}")
+                if s.mf_missing:
+                    l.append(f"  - manifest entries: {[f'vol{mv}.difftar' for mv in s.mf_missing]}")
+            return l
+
+        def missing_difftars(bsets):
+            """
+            Check for and print missing volumes
+            """
+            l = []
+            for bset in bsets:
+                if missing := missing_to_log_info(bset):
+                    l.extend(missing)
+            return l
 
         if self.local_orphaned_sig_names:
             log.Warn(
-                _("Warning, found the following local orphaned signature file(s):")
+                _("WARNING. found the following local orphaned signature file(s):")
                 + "\n"
                 + "\n".join(map(os.fsdecode, self.local_orphaned_sig_names)),
                 log.WarningCode.orphaned_sig,
@@ -850,7 +922,7 @@ class CollectionsStatus(object):
 
         if self.remote_orphaned_sig_names:
             log.Warn(
-                _("Warning, found the following remote orphaned signature file(s):")
+                _("WARNING. found the following remote orphaned signature file(s):")
                 + "\n"
                 + "\n".join(map(os.fsdecode, self.remote_orphaned_sig_names)),
                 log.WarningCode.orphaned_sig,
@@ -858,24 +930,43 @@ class CollectionsStatus(object):
 
         if self.all_sig_chains and sig_chain_warning and not self.matched_chain_pair:
             log.Warn(
-                _("Warning, found signatures but no corresponding " "backup files"),
+                _("WARNING. found signatures but no corresponding backup files"),
                 log.WarningCode.unmatched_sig,
             )
 
-        if self.incomplete_backup_sets:
-            log.Warn(
-                _("Warning, found incomplete backup sets, probably left " "from aborted session"),
-                log.WarningCode.incomplete_backup,
-            )
-
         if self.orphaned_backup_sets:
             log.Warn(
-                _("Warning, found the following orphaned backup file(s):")
+                _("WARNING. found the following orphaned backup file(s):")
                 + "\n"
                 + "\n".join(map(str, self.orphaned_backup_sets)),
                 log.WarningCode.orphaned_backup,
             )
 
+        if self.missing_difftar_sets:
+
+            log.Warn(
+                _("WARNING. found missing difftar(s) in backup sets, possibly left from aborted session:\n")
+                + "\n".join(missing_difftars(self.missing_difftar_sets)),
+                log.WarningCode.incomplete_backup,
+            )
+
+            last_backup_chain = self.all_backup_chains[-1]
+            for bset in last_backup_chain.get_all_sets():
+                if bset.is_missing():
+                    self.last_chain_missing_difftars = True
+
+            if config.action == "inc" and self.last_chain_missing_difftars:
+                if self.first:
+                    log.FatalError(
+                        "ERROR, the last backup chain has missing difftar volumes as above.\n"
+                        "You must run a full backup as the next backup."
+                    )
+                else:
+                    log.Error(
+                        "ERROR, the last backup chain has missing difftar volumes as above.\n"
+                        "You must run a full backup as the next backup."
+                    )
+
     def get_backup_chains(self, filename_list):
         """
         Split given filename_list into chains
@@ -908,7 +999,8 @@ class CollectionsStatus(object):
 
         for f in filename_list:
             add_to_sets(f)
-        sets, incomplete_sets = self.get_sorted_sets(sets)
+
+        sets, incomplete_sets, missing_difftar_sets = self.get_sorted_sets(sets)
 
         chains, orphaned_sets = [], []
 
@@ -922,7 +1014,7 @@ class CollectionsStatus(object):
                 chains.append(new_chain)
                 log.Debug(_("Found backup chain %s") % (new_chain.short_desc()))
             else:
-                assert set.type == "inc"
+                assert set.type == "inc", f"Expected incremental set type 'inc', got {set.type!r} for set {set}"
                 for chain in chains:
                     if chain.add_inc(set):
                         log.Debug(_("Added set %s to pre-existing chain %s") % (set.get_timestr(), chain.short_desc()))
@@ -933,22 +1025,27 @@ class CollectionsStatus(object):
 
         for s in sets:
             add_to_chains(s)
-        return chains, orphaned_sets, incomplete_sets
+        return chains, orphaned_sets, incomplete_sets, missing_difftar_sets
 
     def get_sorted_sets(self, set_list):
         """
-        Sort set list by end time, return (sorted list, incomplete)
+        Sort set list by end time, return (sorted list, incomplete, missing_difftar_sets)
         """
-        time_set_pairs, incomplete_sets = [], []
+        time_set_pairs = []
+        incomplete_sets = []
+        missing_difftar_sets = []
         for set in set_list:  # pylint: disable=redefined-builtin
+            set.get_missing()
             if not set.is_complete():
                 incomplete_sets.append(set)
-            elif set.type == "full":
+            if set.is_missing():
+                missing_difftar_sets.append(set)
+            if set.type == "full":
                 time_set_pairs.append((set.time, set))
             else:
                 time_set_pairs.append((set.end_time, set))
         time_set_pairs.sort(key=lambda x: x[0])
-        return [p[1] for p in time_set_pairs], incomplete_sets
+        return [p[1] for p in time_set_pairs], incomplete_sets, missing_difftar_sets
 
     def get_signature_chains(self, local, filelist=None):
         """
@@ -983,7 +1080,9 @@ class CollectionsStatus(object):
             if pr:
                 if pr.type == "full-sig":
                     new_chain = get_new_sigchain()
-                    assert new_chain.add_filename(filename, pr)
+                    assert new_chain.add_filename(
+                        filename, pr
+                    ), f"Failed to add signature file {os.fsdecode(filename)} to new signature chain"
                     chains.append(new_chain)
                 elif pr.type == "new-sig":
                     new_sig_filenames.append(filename)
@@ -999,7 +1098,8 @@ class CollectionsStatus(object):
                 orphaned_filenames.append(sig_filename)
         return chains, orphaned_filenames
 
-    def get_sorted_chains(self, chain_list):
+    @staticmethod
+    def get_sorted_chains(chain_list):
         """
         Return chains sorted by end_time.  If tie, local goes last
         """
@@ -1019,7 +1119,7 @@ class CollectionsStatus(object):
             if len(chain_list) == 1:
                 sorted_chain_list.append(chain_list[0])
             else:
-                assert len(chain_list) == 2
+                assert len(chain_list) == 2, f"Expected exactly 2 chains with equal end_time, got {len(chain_list)}"
                 if chain_list[0].backend:  # is remote, goes first
                     sorted_chain_list.append(chain_list[0])
                     sorted_chain_list.append(chain_list[1])
@@ -1093,10 +1193,12 @@ class CollectionsStatus(object):
         recognizable as a duplicity file, but isn't part of some
         complete backup set, or current signature chain.
         """
-        assert self.values_set
+        assert (
+            self.values_set
+        ), "CollectionsStatus values must be initialized via set_values() before calling get_extraneous()"
         local_filenames = []
         remote_filenames = []
-        ext_containers = self.orphaned_backup_sets + self.incomplete_backup_sets
+        ext_containers = self.orphaned_backup_sets + self.incomplete_backup_sets + self.missing_difftar_sets
         for set_or_chain in ext_containers:
             if set_or_chain.backend:
                 remote_filenames.extend(set_or_chain.get_filenames())
@@ -1106,7 +1208,8 @@ class CollectionsStatus(object):
         remote_filenames += self.remote_orphaned_sig_names
         return local_filenames, remote_filenames
 
-    def sort_sets(self, setlist):
+    @staticmethod
+    def sort_sets(setlist):
         """Return new list containing same elems of setlist, sorted by time"""
         pairs = sorted([(s.get_time(), s) for s in setlist])
         return [p[1] for p in pairs]
@@ -1121,10 +1224,12 @@ class CollectionsStatus(object):
         than t, and set B is an incremental based on A which is newer
         than t, then the time of set A will not be returned.
         """
-        assert self.values_set
+        assert self.values_set, "CollectionsStatus values must be set before calling get_chains_older_than()"
         old_chains = []
         for chain in self.all_backup_chains:
-            if chain.end_time < t and (not self.matched_chain_pair or chain is not self.matched_chain_pair[1]):
+            if chain.end_time < t and (
+                not self.matched_chain_pair or chain.end_time != self.matched_chain_pair[1].end_time
+            ):
                 # don't delete the active (matched) chain
                 old_chains.append(chain)
         return old_chains
@@ -1139,10 +1244,12 @@ class CollectionsStatus(object):
         than t, and set B is an incremental based on A which is newer
         than t, then the time of set A will not be returned.
         """
-        assert self.values_set
+        assert self.values_set, "CollectionsStatus values must be set before calling get_signature_chains_older_than()"
         old_chains = []
         for chain in self.all_sig_chains:
-            if chain.end_time < t and (not self.matched_chain_pair or chain is not self.matched_chain_pair[0]):
+            if chain.end_time < t and (
+                not self.matched_chain_pair or chain.end_time != self.matched_chain_pair[0].end_time
+            ):
                 # don't delete the active (matched) chain
                 old_chains.append(chain)
         return old_chains
@@ -1181,8 +1288,8 @@ class CollectionsStatus(object):
         a valid input). Thus the second-to-last is obtained with n=2
         rather than n=1.
         """
-        assert self.values_set
-        assert n > 0
+        assert self.values_set, "CollectionsStatus values must be set before calling get_nth_last_backup_chain()"
+        assert n > 0, f"n must be > 0 (1 = latest chain); got n={n}"
 
         if len(self.all_backup_chains) < n:
             return None
@@ -1216,7 +1323,7 @@ class CollectionsStatus(object):
         returns the times of sets which are old but part of the chains
         where the newer end of the chain is newer than t.
         """
-        assert self.values_set
+        assert self.values_set, "CollectionsStatus values must be set before calling get_older_than_required()"
         new_chains = [c for c in self.all_backup_chains if c.end_time >= t]
         result_sets = []
         for chain in new_chains:
@@ -1229,9 +1336,10 @@ class CollectionsStatus(object):
         Returns time line of specified file changed
         """
         # quick fix to spaces in filepath
-        modified_filepath = filepath
-        if " " in str(filepath):
-            modified_filepath = '"' + filepath.replace(" ", r"\x20") + '"'
+        modified_filepath = os.fsdecode(filepath)
+        if " " in modified_filepath:
+            modified_filepath = '"' + modified_filepath.replace(" ", r"\x20") + '"'
+        modified_filepath = os.fsencode(modified_filepath)
 
         if not self.matched_chain_pair:
             return ""
@@ -1240,7 +1348,6 @@ class CollectionsStatus(object):
         specified_file_backup_set = []
         specified_file_backup_type = []
 
-        modified_filepath = os.fsencode(modified_filepath)
         for bs in all_backup_set:
             filelist = [fileinfo[1] for fileinfo in bs.get_files_changed()]
             if modified_filepath in filelist:
@@ -1338,3 +1445,55 @@ class BackupSetChangesStatus(object):
             + ["-------------------------"]
         )
         return "\n".join(l)
+
+
+def PrintCollectionStatus(col_stats, force_print=False):
+    """
+    Prints a collection status to the log.
+    """
+    Log(
+        str(col_stats),
+        8,
+        InfoCode.collection_status,
+        "\n" + "\n".join(col_stats.to_log_info()),
+        force_print,
+    )
+
+
+def PrintCollectionErrors(missing, force_print=True):
+    """
+    Prints a collection status to the log.
+    """
+    Log(
+        "\n" + "\n".join(missing),
+        8,
+        InfoCode.collection_status,
+        None,
+        force_print,
+    )
+
+
+def PrintCollectionFileChangedStatus(col_stats, filepath, force_print=False):
+    """
+    Prints a collection status to the log.
+    """
+    Log(
+        str(col_stats.get_file_changed_record(filepath)),
+        8,
+        InfoCode.collection_status,
+        None,
+        force_print,
+    )
+
+
+def PrintCollectionChangesInSet(col_stats, set_index, force_print=False):
+    """
+    Prints changes in the specified set to the log.
+    """
+    Log(
+        str(col_stats.get_all_file_changed_records(set_index)),
+        8,
+        InfoCode.collection_status,
+        None,
+        force_print,
+    )
