@@ -2,8 +2,8 @@
 #
 # duplicity -- Encrypted bandwidth efficient backup
 #
-# Copyright 2002 Ben Escoto <ben@emerose.org>
-# Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
+# Copyright 2002 Ben Escoto
+# Copyright 2007 Kenneth Loafman
 #
 # This file is part of duplicity.
 #
@@ -36,26 +36,30 @@ import platform
 import resource
 import sys
 import time
+from dataclasses import dataclass
 from textwrap import dedent
 from typing import Dict
 
-from duplicity import __version__
-from duplicity import backend_pool
-from duplicity import cli_main
-from duplicity import config
-from duplicity import diffdir
-from duplicity import dup_collections
-from duplicity import dup_temp
-from duplicity import dup_time
-from duplicity import file_naming
-from duplicity import gpg
-from duplicity import log
-from duplicity import manifest
-from duplicity import patchdir
-from duplicity import path
-from duplicity import progress
-from duplicity import tempdir
-from duplicity import util
+from duplicity import (
+    __version__,
+    backend,
+    backend_pool,
+    cli_main,
+    config,
+    diffdir,
+    dup_collections,
+    dup_temp,
+    dup_time,
+    file_naming,
+    gpg,
+    log,
+    manifest,
+    patchdir,
+    path,
+    progress,
+    tempdir,
+    util,
+)
 from duplicity.errors import BadVolumeException
 
 # If exit_val is not None, exit with given value at end.
@@ -68,6 +72,7 @@ def getpass_safe(message):
     return getpass.getpass(message)
 
 
+# TODO: Simplify and refactor: https://gitlab.com/duplicity/duplicity/-/merge_requests/288#note_2406527475
 def get_passphrase(n, action, for_signing=False):
     """
     Check to make sure passphrase is indeed needed, then get
@@ -137,7 +142,7 @@ def get_passphrase(n, action, for_signing=False):
         "list-current-files",
         "remove-all-but-n-full",
         "remove-all-inc-of-but-n-full",
-        "remove-old",
+        "remove-older-than",
     ]:
         return ""
 
@@ -150,6 +155,14 @@ def get_passphrase(n, action, for_signing=False):
     ):
         return ""
 
+    elif (
+        (config.gpg_profile.recipients or config.gpg_profile.hidden_recipients)
+        and config.metadata_sync_mode == "partial"
+        and action in ["full"]
+    ):
+        log.Info(_("Skipping passphrase input for full backup with encryption keys."))
+        return ""
+
     # Finally, ask the user for the passphrase
     else:
         log.Info(_("PASSPHRASE variable not set, asking user."))
@@ -254,7 +267,7 @@ def restart_position_iterator(tarblock_iter):
                     break
             if tarblock_iter.previous_index > last_index:
                 log.Warn(
-                    _("File %s complete in backup set.\n" "Continuing restart on file %s.")
+                    _("File %s complete in backup set.\nContinuing restart on file %s.")
                     % (util.uindex(last_index), util.uindex(tarblock_iter.previous_index)),
                     log.ErrorCode.restart_file_not_found,
                 )
@@ -264,7 +277,7 @@ def restart_position_iterator(tarblock_iter):
             iter_result = next(tarblock_iter)
     except StopIteration:
         log.Warn(
-            _("File %s missing in backup set.\n" "Continuing restart on file %s.")
+            _("File %s missing in backup set.\nContinuing restart on file %s.")
             % (util.uindex(last_index), util.uindex(tarblock_iter.previous_index)),
             log.ErrorCode.restart_file_not_found,
         )
@@ -349,7 +362,7 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
         vol1_filename = file_naming.get(backup_type, 1, encrypted=config.encryption, gzipped=config.compression)
         if vol1_filename != backup_set.volume_name_dict[1]:
             log.FatalError(
-                _("Restarting backup, but current encryption " "settings do not match original settings"),
+                _("Restarting backup, but current encryption settings do not match original settings"),
                 log.ErrorCode.enryption_mismatch,
             )
 
@@ -366,7 +379,8 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
         transfer_success = False
         manifest_written = False
 
-    def collect_put_results(bytes_written: int, backend_pooler, command2vol_map: Dict[int, CommandMetaData]):
+    def collect_put_results(backend_pooler, command2vol_map: Dict[int, CommandMetaData]):
+        bytes_written = 0
         for result in backend_pooler.results_since_last_call():
             track_id = result.track_id
             size = result.result
@@ -382,6 +396,7 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
                 f"Transfer of {command2vol_map[track_id].path_obj.get_filename()} with id {track_id} and size "
                 f"{size} took {result.get_runtime()}"
             )
+        return bytes_written
 
     def write_manifest_in_sequence(mf, mf_file, command2vol_map: Dict[int, CommandMetaData]):
         """
@@ -442,7 +457,7 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
     backend_pooler = None
     command2vol_map: Dict[int, CommandMetaData] = {}
     if config.concurrency > 0:
-        backend_pooler = backend_pool.BackendPool(backend.backend.parsed_url.url_string, processes=config.concurrency)
+        backend_pooler = backend_pool.BackendPool(config.target_url, processes=config.concurrency)
 
     while not at_end:
         # set up iterator
@@ -478,7 +493,7 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
             # if nothing changed, skip upload if configured.
             msg = _("Skipped volume upload, as effectivly nothing has changed")
             log.Progress(msg, diffdir.stats.SourceFileSize)
-            log.Notice(_(msg))
+            log.Notice(msg)
             config.skipped_inc = True
             tdp.delete()
             continue
@@ -489,7 +504,7 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
                 progress.report_transfer(0, tdp.getsize())
                 track_id = backend_pooler.command_throttled(backend.put_validated.__name__, args=(tdp, dest_filename))
                 command2vol_map[track_id] = CommandMetaData(vol_num, tdp, vi)
-                collect_put_results(bytes_written, backend_pooler, command2vol_map)
+                bytes_written += collect_put_results(backend_pooler, command2vol_map)
                 write_manifest_in_sequence(mf, man_outfp, command2vol_map)
             except (Exception, SystemExit) as e:
                 # ensure pool processes terminate clean
@@ -519,7 +534,7 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
             # wait for background commands, collect some stats and shutdown clean.
             log.Debug("Collecting remaining results from backend pool.")
             while True and backend_pooler:
-                collect_put_results(bytes_written, backend_pooler, command2vol_map)
+                bytes_written += collect_put_results(backend_pooler, command2vol_map)
                 write_manifest_in_sequence(mf, man_outfp, command2vol_map)
                 if backend_pooler.get_queue_length() == 0:
                     break
@@ -565,7 +580,7 @@ def get_man_fileobj(backup_type):
     @rtype: fileobj
     @return: fileobj opened for writing
     """
-    assert backup_type == "full" or backup_type == "inc"
+    assert backup_type == "full" or backup_type == "inc", f"backup_type must be 'full' or 'inc', got {backup_type!r}"
 
     part_man_filename = file_naming.get(backup_type, manifest=True, partial=True)
     perm_man_filename = file_naming.get(backup_type, manifest=True)
@@ -590,7 +605,7 @@ def get_sig_fileobj(sig_type):
     @rtype: fileobj
     @return: fileobj opened for writing
     """
-    assert sig_type in ["full-sig", "new-sig"]
+    assert sig_type in ["full-sig", "new-sig"], f"sig_type must be 'full-sig' or 'new-sig', got {sig_type!r}"
 
     part_sig_filename = file_naming.get(sig_type, gzipped=False, partial=True)
     perm_sig_filename = file_naming.get(sig_type, gzipped=True)
@@ -615,7 +630,7 @@ def get_stat_fileobj(stat_type):
     @rtype: fileobj
     @return: fileobj opened for writing
     """
-    assert stat_type in ["full-stat", "inc-stat"]
+    assert stat_type in ["full-stat", "inc-stat"], f"stat_type must be 'full-stat' or 'inc-stat', got {stat_type!r}"
 
     part_stat_filename = file_naming.get(stat_type, gzipped=False, partial=True)
     perm_stat_filename = file_naming.get(stat_type, gzipped=True)
@@ -673,7 +688,7 @@ def full_backup(col_stats):
             # Terminate the background thread now, if any
             progress.progress_thread.finished = True
             progress.progress_thread.join()
-            log.TransferProgress(
+            progress.TransferProgress(
                 100.0,
                 0,
                 progress.tracker.total_bytecount,
@@ -783,7 +798,7 @@ def incremental_backup(sig_chain, col_stats=None):
             # Terminate the background thread now, if any
             progress.progress_thread.finished = True
             progress.progress_thread.join()
-            log.TransferProgress(
+            progress.TransferProgress(
                 100.0,
                 0,
                 progress.tracker.total_bytecount,
@@ -879,7 +894,10 @@ def restore_get_patched_rop_iter(col_stats):
         index = ()
     time = config.restore_time or dup_time.curtime
     backup_chain = col_stats.get_backup_chain_at_time(time)
-    assert backup_chain, col_stats.all_backup_chains
+    assert backup_chain, (
+        f"No backup chain found for restore time {dup_time.timetostring(time)}; "
+        f"available chains: {col_stats.all_backup_chains}"
+    )
     backup_setlist = backup_chain.get_sets_at_time(time)
     num_vols = 0
     for s in backup_setlist:
@@ -887,13 +905,17 @@ def restore_get_patched_rop_iter(col_stats):
     cur_vol = [0]
 
     def get_fileobj_iter(backup_set):
-        """Get file object iterator from backup_set contain given index"""
+        """
+        Get file object iterator from backup_set contain given index
+        """
         manifest = backup_set.get_manifest()
         volumes = manifest.get_containing_volumes(index)
         for vol_num in volumes:
             try:
                 fobj = restore_get_enc_fileobj(
-                    backup_set.backend, backup_set.volume_name_dict[vol_num], manifest.volume_info_dict[vol_num]
+                    backup_set.backend,
+                    backup_set.volume_name_dict[vol_num],
+                    manifest.volume_info_dict[vol_num],
                 )
                 if fobj is not None:
                     yield fobj
@@ -909,7 +931,18 @@ def restore_get_patched_rop_iter(col_stats):
             manifest = backup_set.get_manifest()
             volumes = manifest.get_containing_volumes(index)
             for vol_num in volumes:
-                file_names.append(backup_set.volume_name_dict[vol_num])
+                try:
+                    file_names.append(backup_set.volume_name_dict[vol_num])
+                except KeyError as e:
+                    if config.ignore_errors:
+                        log.Warn(f"difftar volume {vol_num} not found, ignoring per --ignore-errors.")
+                    else:
+                        log.FatalError(
+                            f"difftar volume {vol_num} not found in backup.\n"
+                            f"Please check your archive and try again.\n"
+                            f"Or use --ignore-errors to ignore this error.\n"
+                            f"Error: {e}"
+                        )
         if config.dry_run:
             log.Notice("Required volumes to restore:\n\t" + "\n\t".join(file_name.decode() for file_name in file_names))
             return None
@@ -954,7 +987,7 @@ def restore_get_enc_fileobj(backend, filename, volume_info):
         if config.ignore_errors:
             exc = BadVolumeException(f"Hash mismatch for: {os.fsdecode(filename)}")
             log.Warn(
-                _("IGNORED_ERROR: Warning: ignoring error as requested: %s: %s")
+                _("IGNORED_ERROR: WARNING: ignoring error as requested: %s: %s")
                 % (exc.__class__.__name__, util.uexc(exc))
             )
             # Do not try to actually read it as it is corrupted!
@@ -1068,11 +1101,8 @@ def cleanup(col_stats):
                     pass
     else:
         log.Notice(
-            _("Found the following file(s) to delete:")
-            + "\n"
-            + filestr
-            + "\n"
-            + _("Run duplicity again with the --force option to actually delete.")
+            f"{_('Found the following file(s) to delete:')}\n{filestr}\n"
+            "Run duplicity again with the --force option to actually delete."
         )
 
 
@@ -1086,7 +1116,9 @@ def remove_all_but_n_full(col_stats):
     @rtype: void
     @return: void
     """
-    assert config.keep_chains is not None
+    assert (
+        config.keep_chains is not None
+    ), "config.keep_chains (from --keep-chains) must be set before removing old backups"
 
     config.remove_time = col_stats.get_nth_last_full_backup_time(config.keep_chains)
 
@@ -1103,7 +1135,7 @@ def remove_old(col_stats):
     @rtype: void
     @return: void
     """
-    assert config.remove_time is not None
+    assert config.remove_time is not None, "config.remove_time must be set before removing old backups"
 
     def set_times_str(setlist):
         """Return string listing times of sets in setlist"""
@@ -1188,7 +1220,7 @@ def sync_archive(col_stats):
     @rtype: void
     @return: void
     """
-    suffixes = [b".g", b".gpg", b".z", b".gz", b".part"]
+    suffixes = [b".g", b".gpg", b".p7m", b".z", b".gz", b".part"]
 
     def is_needed(filename):
         """Indicates if the metadata file should be synced.
@@ -1200,7 +1232,7 @@ def sync_archive(col_stats):
         """
         if config.metadata_sync_mode == "full":
             return True
-        assert config.metadata_sync_mode == "partial"
+        assert config.metadata_sync_mode == "partial", "metadata_sync_mode must be 'partial' when not in full sync mode"
         parsed = file_naming.parse(filename)
         try:
             target_chain = col_stats.get_backup_chain_at_time(config.restore_time or dup_time.curtime)
@@ -1342,6 +1374,10 @@ def sync_archive(col_stats):
         tdp.setdata()
         tdp.move(config.archive_dir_path.append(loc_name))
 
+    # skip if remote access is not configured
+    if not config.check_remote:
+        return
+
     # get remote metafile list
     remlist = config.backend.list()
     remote_metafiles, ignored, rem_needpass = get_metafiles(remlist)
@@ -1417,11 +1453,10 @@ def check_last_manifest(col_stats):
     @rtype: void
     @return: void
     """
-    assert col_stats.all_backup_chains
+    assert col_stats.all_backup_chains, "No backup chains found; cannot check last manifest without any chains present"
     last_backup_set = col_stats.all_backup_chains[-1].get_last()
     # check remote manifest only if we can decrypt it (see #1729796)
-    check_remote = not config.encryption or config.gpg_profile.passphrase
-    last_backup_set.check_manifests(check_remote=check_remote)
+    last_backup_set.check_manifests(check_remote=config.check_remote)
 
 
 def check_resources(action):
@@ -1483,7 +1518,8 @@ def log_startup_parms(verbosity=log.INFO):
     """
     log.Log("=" * 80, verbosity)
     log.Log(f"duplicity {__version__}", verbosity)
-    log.Log(f"Args: {' '.join([os.fsdecode(arg) for arg in sys.argv])}", verbosity)
+    cmd_line = " ".join([os.fsdecode(arg) for arg in sys.argv])
+    log.Log(f"Args: {backend.Backend.munge_password(cmd_line)}", verbosity)
     log.Log(" ".join(platform.uname()), verbosity)
     log.Log(f"{sys.executable or sys.platform} {sys.version}", verbosity)
     log.Log("=" * 80, verbosity)
@@ -1496,6 +1532,7 @@ class Restart(object):
     """
 
     def __init__(self, last_backup):
+        self.time = None
         self.type = None
         self.start_time = None
         self.end_time = None
@@ -1566,19 +1603,20 @@ def main():
     Start/end here
     """
     # per bug https://bugs.launchpad.net/duplicity/+bug/931175
-    # duplicity crashes when PYTHONOPTIMIZE is set, so check
+    # duplicity crashes when python optimization is set, so check
     # and refuse to run if it is set.
-    if "PYTHONOPTIMIZE" in os.environ:
+    if sys.flags.optimize:
         log.FatalError(
             dedent(
                 _(
                     """\
-                PYTHONOPTIMIZE in the environment causes duplicity to fail to
-                recognize its own backups.  Please remove PYTHONOPTIMIZE from
-                the environment and rerun the backup.
+                    Setting python optimization causes duplicity to fail to
+                    recognize its own backups.  Please remove PYTHONOPTIMIZE
+                    from the environment or -O and -OO from the commandline,
+                    then rerun the backup.
 
-                See https://bugs.launchpad.net/duplicity/+bug/931175
-                """
+                    See https://bugs.launchpad.net/duplicity/+bug/931175
+                    """
                 )
             ),
             log.ErrorCode.pythonoptimize_set,
@@ -1620,11 +1658,16 @@ def do_backup(action):
     check_resources(action)
 
     # get current collection status
-    col_stats = dup_collections.CollectionsStatus(config.backend, config.archive_dir_path, action).set_values()
+    col_stats = dup_collections.CollectionsStatus(
+        config.backend,
+        config.archive_dir_path,
+        first=True,
+    ).set_values()
 
     # check archive synch with remote, fix if needed
     if action not in [
         "collection-status",
+        "full",
         "remove-all-but-n-full",
         "remove-all-inc-of-but-n-full",
         "remove-old",
@@ -1659,7 +1702,8 @@ def do_backup(action):
                     log.Notice(_(f"Cleaning up previous partial {action} backup set, restarting."))
                     last_backup.delete()
                     col_stats = dup_collections.CollectionsStatus(
-                        config.backend, config.archive_dir_path, action
+                        config.backend,
+                        config.archive_dir_path,
                     ).set_values()
                     continue
             break
@@ -1679,7 +1723,7 @@ def do_backup(action):
     ):
         log.Notice(_("Last full backup is too old, forcing full backup"))
         action = "full"
-    log.PrintCollectionStatus(col_stats)
+    dup_collections.PrintCollectionStatus(col_stats)
 
     # get the passphrase if we need to based on action/options
     config.gpg_profile.passphrase = get_passphrase(1, action)
@@ -1694,20 +1738,20 @@ def do_backup(action):
         if config.show_changes_in_set is not None:
             if not config.jsonstat:
                 # print classic stats
-                log.PrintCollectionChangesInSet(col_stats, config.show_changes_in_set, True)
+                dup_collections.PrintCollectionChangesInSet(col_stats, config.show_changes_in_set, True)
             else:
                 # print json stat
                 json_stat = col_stats.get_changes_in_set_json(config.show_changes_in_set)
                 log.Log(str(json_stat), 8, log.InfoCode.collection_status, None, True)
         elif not config.file_changed:
-            log.PrintCollectionStatus(col_stats, True)
+            dup_collections.PrintCollectionStatus(col_stats, True)
         else:
-            log.PrintCollectionFileChangedStatus(col_stats, config.file_changed, True)
+            dup_collections.PrintCollectionFileChangedStatus(col_stats, config.file_changed, True)
     elif action == "cleanup":
         cleanup(col_stats)
     elif action == "remove-older-than":
         remove_old(col_stats)
-    elif action == "remove-all-but-n-full" or action == "remove-all-inc-of-but-n-full":
+    elif action in ["remove-all-but-n-full", "remove-all-inc-of-but-n-full"]:
         remove_all_but_n_full(col_stats)
     elif action == "sync":
         sync_archive(col_stats)
@@ -1754,6 +1798,12 @@ def do_backup(action):
                         check_last_manifest(col_stats)  # not needed for full backups
                 incremental_backup(sig_chain, col_stats)
 
+        if action in ["full", "inc"] and not config.check_remote:
+            dup_collections.CollectionsStatus(
+                config.backend,
+                config.archive_dir_path,
+            ).set_values()
+
     config.backend.close()
     log.shutdown()
     if exit_val is not None:
