@@ -1,7 +1,7 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
-# Copyright 2002 Ben Escoto <ben@emerose.org>
-# Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
+# Copyright 2002 Ben Escoto
+# Copyright 2007 Kenneth Loafman
 #
 # This file is part of duplicity.
 #
@@ -26,14 +26,15 @@ which is written in C.  The goal was to use C as little as possible...
 
 """
 
-import os
-from . import _librsync
-import types
 import array
+import os
 
-if os.environ.get('READTHEDOCS') == 'True':
-    import mock
+from . import _librsync
+
+if os.environ.get("READTHEDOCS") == "True":
+    import unittest.mock as mock
     import duplicity
+
     duplicity._librsync = mock.MagicMock()
 
 blocksize = _librsync.RS_JOB_BLOCKSIZE
@@ -48,11 +49,13 @@ class librsyncError(Exception):
     but this scheme was easier.
 
     """
+
     pass
 
 
-class LikeFile:
+class LikeFile(object):
     """File-like object used by SigFile, DeltaFile, and PatchFile"""
+
     mode = "rb"
 
     # This will be replaced in subclasses by an object with
@@ -64,8 +67,8 @@ class LikeFile:
         self.check_file(infile, need_seek)
         self.infile = infile
         self.closed = self.infile_closed = None
-        self.inbuf = ""
-        self.outbuf = array.array('c')
+        self.inbuf = b""
+        self.outbuf = array.array("b")
         self.eof = self.infile_eof = None
 
     def check_file(self, file, need_seek=None):
@@ -88,7 +91,7 @@ class LikeFile:
                 self._add_to_outbuf_once()
             real_len = min(length, len(self.outbuf))
 
-        return_val = self.outbuf[:real_len].tostring()
+        return_val = self.outbuf[:real_len].tobytes()
         del self.outbuf[:real_len]
         return return_val
 
@@ -101,16 +104,16 @@ class LikeFile:
         except _librsync.librsyncError as e:
             raise librsyncError(str(e))
         self.inbuf = self.inbuf[len_inbuf_read:]
-        self.outbuf.fromstring(cycle_out)
+        self.outbuf.frombytes(cycle_out)
 
     def _add_to_inbuf(self):
         """Make sure len(self.inbuf) >= blocksize"""
-        assert not self.infile_eof
+        assert not self.infile_eof, "infile_eof already set"
         while len(self.inbuf) < blocksize:
             new_in = self.infile.read(blocksize)
             if not new_in:
                 self.infile_eof = 1
-                assert not self.infile.close()
+                assert not self.infile.close(), "self.infile failed to close"
                 self.infile_closed = 1
                 break
             self.inbuf += new_in
@@ -118,12 +121,13 @@ class LikeFile:
     def close(self):
         """Close infile"""
         if not self.infile_closed:
-            assert not self.infile.close()
+            assert not self.infile.close(), "self.infile failed to close"
         self.closed = 1
 
 
 class SigFile(LikeFile):
     """File-like object which incrementally generates a librsync signature"""
+
     def __init__(self, infile, blocksize=_librsync.RS_DEFAULT_BLOCK_LEN):
         """SigFile initializer - takes basis file
 
@@ -140,6 +144,7 @@ class SigFile(LikeFile):
 
 class DeltaFile(LikeFile):
     """File-like object which incrementally generates a librsync delta"""
+
     def __init__(self, signature, new_file):
         """DeltaFile initializer - call with signature and new file
 
@@ -149,12 +154,12 @@ class DeltaFile(LikeFile):
 
         """
         LikeFile.__init__(self, new_file)
-        if isinstance(signature, types.StringType):
+        if isinstance(signature, bytes):
             sig_string = signature
         else:
             self.check_file(signature)
             sig_string = signature.read()
-            assert not signature.close()
+            assert not signature.close(), "signature failed to close"
         try:
             self.maker = _librsync.new_deltamaker(sig_string)
         except _librsync.librsyncError as e:
@@ -163,6 +168,7 @@ class DeltaFile(LikeFile):
 
 class PatchedFile(LikeFile):
     """File-like object which applies a librsync delta incrementally"""
+
     def __init__(self, basis_file, delta_file):
         """PatchedFile initializer - call with basis delta
 
@@ -172,30 +178,37 @@ class PatchedFile(LikeFile):
 
         """
         LikeFile.__init__(self, delta_file)
-        if not isinstance(basis_file, types.FileType):
-            """ tempfile.TemporaryFile() only guarantees a true file
+        try:
+            basis_file.fileno()
+        except Exception as e:
+            """tempfile.TemporaryFile() only guarantees a true file
             object on posix platforms. on cygwin/windows a file-like
             object whose file attribute is the underlying true file
             object is returned.
             """
-            if hasattr(basis_file, 'file') and isinstance(basis_file.file, types.FileType):
+            if hasattr(basis_file, "file") and hasattr(basis_file.file, "fileno"):
                 basis_file = basis_file.file
             else:
-                raise TypeError(_("basis_file must be a (true) file or an object whose "
-                                  "file attribute is the underlying true file object"))
+                raise TypeError(
+                    _(
+                        "basis_file must be a (true) file or an object whose "
+                        "file attribute is the underlying true file object"
+                    )
+                )
         try:
             self.maker = _librsync.new_patchmaker(basis_file)
         except _librsync.librsyncError as e:
             raise librsyncError(str(e))
 
 
-class SigGenerator:
+class SigGenerator(object):
     """Calculate signature.
 
     Input and output is same as SigFile, but the interface is like md5
     module, not filelike object
 
     """
+
     def __init__(self, blocksize=_librsync.RS_DEFAULT_BLOCK_LEN):
         """Return new signature instance"""
         try:
@@ -203,7 +216,7 @@ class SigGenerator:
         except _librsync.librsyncError as e:
             raise librsyncError(str(e))
         self.gotsig = None
-        self.buffer = ""
+        self.buffer = b""
         self.sigstring_list = []
 
     def update(self, buf):
@@ -229,4 +242,4 @@ class SigGenerator:
         """Return signature over given data"""
         while not self.process_buffer():
             pass  # keep running until eof
-        return ''.join(self.sigstring_list)
+        return b"".join(self.sigstring_list)
