@@ -53,7 +53,7 @@ def exception_traceback(limit=50):
     lines.extend(traceback.format_exception_only(type, value))
 
     msg = "Traceback (innermost last):\n"
-    msg = msg + "%-20s %s" % (str.join("", lines[:-1]), lines[-1])
+    msg = msg + f"{str.join('', lines[:-1]):20} {lines[-1]}"
 
     return msg
 
@@ -107,7 +107,7 @@ def maybe_ignore_errors(fn):
         return fn()
     except Exception as e:
         if config.ignore_errors:
-            log.Warn(_("IGNORED_ERROR: Warning: ignoring error as requested: %s: %s") % (e.__class__.__name__, uexc(e)))
+            log.Warn(_("IGNORED_ERROR: WARNING: ignoring error as requested: %s: %s") % (e.__class__.__name__, uexc(e)))
             return None
         else:
             raise
@@ -345,12 +345,12 @@ def start_debugger():
         # ignition
         try:
             pydevd_pycharm.settrace(
-                debug_host,
+                host=debug_host,
                 port=debug_port,
                 suspend=False,
-                stdoutToServer=True,
-                stderrToServer=True,
-                # patch_multiprocessing=True,
+                stdout_to_server=True,
+                stderr_to_server=True,
+                patch_multiprocessing=True,
             )
             log.Info(f"Connection {debug_host}:{debug_port} accepted for debug.")
         except ConnectionRefusedError as e:
